package eu.clarin.cqi.ws.resources;

import eu.clarin.cqi.ws.impl.QueryEngine;
import eu.clarin.cqi.ws.impl.QueryEngineException;
import de.tuebingen.uni.sfs.wlf1.xb.WLData;
import eu.clarin.cqi.client.CqiClientException;
import java.util.HashMap;
import java.util.Map;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;

/**
 *
 * @author akislev
 */
@Path("")
public class CQPResource {

    private String cqiServerHostname;
    private int cqiServerPort;
    private String cqiServerUsername;
    private String cqiServerPassword;
    private int maxQueryResults;

    public void setCqiServerHostname(String cqiServerHostname) {
        this.cqiServerHostname = cqiServerHostname;
    }

    public void setCqiServerPassword(String cqiServerPassword) {
        this.cqiServerPassword = cqiServerPassword;
    }

    public void setCqiServerPort(int cqiServerPort) {
        this.cqiServerPort = cqiServerPort;
    }

    public void setCqiServerUsername(String cqiServerUsername) {
        this.cqiServerUsername = cqiServerUsername;
    }

    public void setMaxQueryResults(int maxQueryResults) {
        this.maxQueryResults = maxQueryResults;
    }

    @GET
    @Path("query")
    @Produces(MediaType.TEXT_XML)
    public WLData getQueryCQP(@QueryParam("query") String query, @DefaultValue("tueba-ddc") @QueryParam("corpus") String corpusName, @DefaultValue("1") @QueryParam("start") int startRecord, @DefaultValue("10") @QueryParam("results") int resultsNumber) {
        if (query == null) {
            throw new ResourceException("\"query\" parameter is not specified");
        }
        return postQueryCQP(query, corpusName, startRecord, resultsNumber);
    }

    @POST
    @Path("query")
    @Consumes("text/query-cqp")
    @Produces("text/tcf+xml")
    public WLData postQueryCQP(String query, @DefaultValue("tueba-ddc") @QueryParam("corpus") final String corpusName, @DefaultValue("1") @QueryParam("start") int startRecord, @DefaultValue("10") @QueryParam("results") int resultsNumber) {
        if (resultsNumber > maxQueryResults || resultsNumber < 1) {
            throw new ResourceException("bad results parameter value");
        }
        try {
            QueryEngine queryEngine = new QueryEngine(cqiServerHostname, cqiServerPort, cqiServerUsername, cqiServerPassword);
            WLData data = queryEngine.query(corpusName.toUpperCase(), query, startRecord, resultsNumber);
            queryEngine.close();
            return data;
        } catch (QueryEngineException ex) {
            throw new ResourceException(ex);
        } catch (CqiClientException ex) {
            throw new ResourceException(ex);
        }
    }
}
